@@ -16,9 +16,8 @@
 #include <string.h>
 #include <sys/types.h>
 
-#include <gio/gio.h>
-
 #include "soup-address.h"
+#include "soup-dns.h"
 #include "soup-enum-types.h"
 #include "soup-marshal.h"
 #include "soup-misc.h"
@@ -39,6 +38,9 @@
  * #SoupAddress represents the address of a TCP connection endpoint:
  * both the IP address and the port. (It is somewhat like an
  * object-oriented version of struct sockaddr.)
+ *
+ * If libsoup was built with IPv6 support, #SoupAddress will allow
+ * both IPv4 and IPv6 addresses.
  **/
 
 enum {
@@ -59,19 +61,22 @@ typedef struct {
 	char *name, *physical;
 	guint port;
 
-	GMutex *lock;
-	GSList *async_lookups;
+	SoupDNSLookup *lookup;
+	guint timeout_id;
 } SoupAddressPrivate;
 #define SOUP_ADDRESS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOUP_TYPE_ADDRESS, SoupAddressPrivate))
 
 /* sockaddr generic macros */
 #define SOUP_SIN(priv) ((struct sockaddr_in *)priv->sockaddr)
+#ifdef HAVE_IPV6
 #define SOUP_SIN6(priv) ((struct sockaddr_in6 *)priv->sockaddr)
+#endif
 
 /* sockaddr family macros */
 #define SOUP_ADDRESS_GET_FAMILY(priv) (priv->sockaddr->sa_family)
 #define SOUP_ADDRESS_SET_FAMILY(priv, family) \
 	(priv->sockaddr->sa_family = family)
+#ifdef HAVE_IPV6
 #define SOUP_ADDRESS_FAMILY_IS_VALID(family) \
 	(family == AF_INET || family == AF_INET6)
 #define SOUP_ADDRESS_FAMILY_SOCKADDR_SIZE(family) \
@@ -80,9 +85,15 @@ typedef struct {
 #define SOUP_ADDRESS_FAMILY_DATA_SIZE(family) \
 	(family == AF_INET ? sizeof (struct in_addr) : \
 			     sizeof (struct in6_addr))
+#else
+#define SOUP_ADDRESS_FAMILY_IS_VALID(family) (family == AF_INET)
+#define SOUP_ADDRESS_FAMILY_SOCKADDR_SIZE(family) sizeof (struct sockaddr_in)
+#define SOUP_ADDRESS_FAMILY_DATA_SIZE(family) sizeof (struct in_addr)
+#endif
 
 /* sockaddr port macros */
 #define SOUP_ADDRESS_PORT_IS_VALID(port) (port >= 0 && port <= 65535)
+#ifdef HAVE_IPV6
 #define SOUP_ADDRESS_GET_PORT(priv) \
 	(priv->sockaddr->sa_family == AF_INET ? \
 		SOUP_SIN(priv)->sin_port : \
@@ -94,12 +105,20 @@ typedef struct {
 	else						\
 		SOUP_SIN6(priv)->sin6_port = port;	\
 	} G_STMT_END
+#else
+#define SOUP_ADDRESS_GET_PORT(priv) (SOUP_SIN(priv)->sin_port)
+#define SOUP_ADDRESS_SET_PORT(priv, port) (SOUP_SIN(priv)->sin_port = port)
+#endif
 
 /* sockaddr data macros */
+#ifdef HAVE_IPV6
 #define SOUP_ADDRESS_GET_DATA(priv) \
 	(priv->sockaddr->sa_family == AF_INET ? \
 		(gpointer)&SOUP_SIN(priv)->sin_addr : \
 		(gpointer)&SOUP_SIN6(priv)->sin6_addr)
+#else
+#define SOUP_ADDRESS_GET_DATA(priv) ((gpointer)&SOUP_SIN(priv)->sin_addr)
+#endif
 #define SOUP_ADDRESS_SET_DATA(priv, data, length) \
 	memcpy (SOUP_ADDRESS_GET_DATA (priv), data, length)
 
@@ -117,9 +136,6 @@ G_DEFINE_TYPE (SoupAddress, soup_address, G_TYPE_OBJECT)
 static void
 soup_address_init (SoupAddress *addr)
 {
-	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
-
-	priv->lock = g_mutex_new ();
 }
 
 static void
@@ -135,7 +151,10 @@ finalize (GObject *object)
 	if (priv->physical)
 		g_free (priv->physical);
 
-	g_mutex_free (priv->lock);
+	if (priv->lookup)
+		soup_dns_lookup_free (priv->lookup);
+	if (priv->timeout_id)
+		g_source_remove (priv->timeout_id);
 
 	G_OBJECT_CLASS (soup_address_parent_class)->finalize (object);
 }
@@ -145,6 +164,8 @@ soup_address_class_init (SoupAddressClass *address_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (address_class);
 
+	soup_dns_init ();
+
 	g_type_class_add_private (address_class, sizeof (SoupAddressPrivate));
 
 	/* virtual method override */
@@ -154,12 +175,6 @@ soup_address_class_init (SoupAddressClass *address_class)
 	object_class->get_property = get_property;
 
 	/* properties */
-	/**
-	 * SOUP_ADDRESS_NAME:
-	 *
-	 * Alias for the #SoupAddress:name property. (The hostname for
-	 * this address.)
-	 **/
 	g_object_class_install_property (
 		object_class, PROP_NAME,
 		g_param_spec_string (SOUP_ADDRESS_NAME,
@@ -167,12 +182,6 @@ soup_address_class_init (SoupAddressClass *address_class)
 				     "Hostname for this address",
 				     NULL,
 				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	/**
-	 * SOUP_ADDRESS_FAMILY:
-	 *
-	 * Alias for the #SoupAddress:family property. (The
-	 * #SoupAddressFamily for this address.)
-	 **/
 	g_object_class_install_property (
 		object_class, PROP_FAMILY,
 		g_param_spec_enum (SOUP_ADDRESS_FAMILY,
@@ -181,12 +190,6 @@ soup_address_class_init (SoupAddressClass *address_class)
 				   SOUP_TYPE_ADDRESS_FAMILY,
 				   SOUP_ADDRESS_FAMILY_INVALID,
 				   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	/**
-	 * SOUP_ADDRESS_PORT:
-	 *
-	 * An alias for the #SoupAddress:port property. (The port for
-	 * this address.)
-	 **/
 	g_object_class_install_property (
 		object_class, PROP_PORT,
 		g_param_spec_int (SOUP_ADDRESS_PORT,
@@ -194,12 +197,6 @@ soup_address_class_init (SoupAddressClass *address_class)
 				  "Port for this address",
 				  -1, 65535, -1,
 				  G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	/**
-	 * SOUP_ADDRESS_PHYSICAL:
-	 *
-	 * An alias for the #SoupAddress:physical property. (The
-	 * stringified IP address for this address.)
-	 **/
 	g_object_class_install_property (
 		object_class, PROP_PHYSICAL,
 		g_param_spec_string (SOUP_ADDRESS_PHYSICAL,
@@ -207,18 +204,21 @@ soup_address_class_init (SoupAddressClass *address_class)
 				     "IP address for this address",
 				     NULL,
 				     G_PARAM_READABLE));
-	/**
-	 * SOUP_ADDRESS_SOCKADDR:
-	 *
-	 * An alias for the #SoupAddress:sockaddr property. (A pointer
-	 * to the struct sockaddr for this address.)
-	 **/
 	g_object_class_install_property (
 		object_class, PROP_SOCKADDR,
 		g_param_spec_pointer (SOUP_ADDRESS_SOCKADDR,
 				      "sockaddr",
 				      "struct sockaddr for this address",
 				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+#ifdef G_OS_WIN32
+	/* This hopefully is a good place to call WSAStartup */
+	{
+		WSADATA wsadata;
+		if (WSAStartup (MAKEWORD (2, 0), &wsadata) != 0)
+			g_error ("Windows Sockets could not be initialized");
+	}
+#endif
 }
 
 static GObject *
@@ -235,7 +235,12 @@ constructor (GType                  type,
 		return NULL;
 	priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 
-	if (!priv->name && !priv->sockaddr) {
+	if (priv->name) {
+		if (!priv->sockaddr)
+			priv->lookup = soup_dns_lookup_name (priv->name);
+	} else if (priv->sockaddr)
+		priv->lookup = soup_dns_lookup_address (priv->sockaddr);
+	else {
 		g_object_unref (addr);
 		return NULL;
 	}
@@ -384,7 +389,10 @@ soup_address_new_from_sockaddr (struct sockaddr *sa, int len)
  * @SOUP_ADDRESS_FAMILY_IPV4: an IPv4 address
  * @SOUP_ADDRESS_FAMILY_IPV6: an IPv6 address
  *
- * The supported address families.
+ * The supported address families. Note that the
+ * %SOUP_ADDRESS_FAMILY_IPV6 constant is available even if libsoup was
+ * built without IPv6 support, but attempting to create an IPv6
+ * address will fail in that case.
  **/
 
 /**
@@ -423,11 +431,6 @@ soup_address_new_any (SoupAddressFamily family, guint port)
  *
  * Returns the hostname associated with @addr.
  *
- * This method is not thread-safe; if you call it while @addr is being
- * resolved in another thread, it may return garbage. You can use
- * soup_address_is_resolved() to safely test whether or not an address
- * is resolved before fetching its name or address.
- *
  * Return value: the hostname, or %NULL if it is not known.
  **/
 const char *
@@ -446,11 +449,6 @@ soup_address_get_name (SoupAddress *addr)
  * Returns the sockaddr associated with @addr, with its length in
  * *@len. If the sockaddr is not yet known, returns %NULL.
  *
- * This method is not thread-safe; if you call it while @addr is being
- * resolved in another thread, it may return garbage. You can use
- * soup_address_is_resolved() to safely test whether or not an address
- * is resolved before fetching its name or address.
- *
  * Return value: the sockaddr, or %NULL
  **/
 struct sockaddr *
@@ -463,24 +461,10 @@ soup_address_get_sockaddr (SoupAddress *addr, int *len)
 
 	if (priv->sockaddr && len)
 		*len = SOUP_ADDRESS_FAMILY_SOCKADDR_SIZE (SOUP_ADDRESS_GET_FAMILY (priv));
+
 	return priv->sockaddr;
 }
 
-static GInetAddress *
-soup_address_make_inet_address (SoupAddress *addr)
-{
-	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
-	GSocketAddress *gsa;
-	GInetAddress *gia;
-
-	gsa = g_socket_address_new_from_native (priv->sockaddr,
-						SOUP_ADDRESS_FAMILY_SOCKADDR_SIZE (SOUP_ADDRESS_GET_FAMILY (priv)));
-	gia = g_inet_socket_address_get_address ((GInetSocketAddress *)gsa);
-	g_object_ref (gia);
-	g_object_unref (gsa);
-	return gia;
-}
-
 /**
  * soup_address_get_physical:
  * @addr: a #SoupAddress
@@ -488,11 +472,6 @@ soup_address_make_inet_address (SoupAddress *addr)
  * Returns the physical address associated with @addr as a string.
  * (Eg, "127.0.0.1"). If the address is not yet known, returns %NULL.
  *
- * This method is not thread-safe; if you call it while @addr is being
- * resolved in another thread, it may return garbage. You can use
- * soup_address_is_resolved() to safely test whether or not an address
- * is resolved before fetching its name or address.
- *
  * Return value: the physical address, or %NULL
  **/
 const char *
@@ -506,13 +485,8 @@ soup_address_get_physical (SoupAddress *addr)
 	if (!priv->sockaddr)
 		return NULL;
 
-	if (!priv->physical) {
-		GInetAddress *gia;
-
-		gia = soup_address_make_inet_address (addr);
-		priv->physical = g_inet_address_to_string (gia);
-		g_object_unref (gia);
-	}
+	if (!priv->physical)
+		priv->physical = soup_dns_ntop (priv->sockaddr);
 
 	return priv->physical;
 }
@@ -534,128 +508,48 @@ soup_address_get_port (SoupAddress *addr)
 }
 
 
-static guint
-update_addrs (SoupAddress *addr, GList *addrs, GError *error)
-{
-	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
-	GInetAddress *gia;
-	GSocketAddress *gsa;
-	gsize len;
-
-	if (error) {
-		if (error->domain == G_IO_ERROR &&
-		    error->code == G_IO_ERROR_CANCELLED)
-			return SOUP_STATUS_CANCELLED;
-		else
-			return SOUP_STATUS_CANT_RESOLVE;
-	} else if (!addrs)
-		return SOUP_STATUS_CANT_RESOLVE;
-	else if (priv->sockaddr)
-		return SOUP_STATUS_OK;
-
-	gia = addrs->data;
-	gsa = g_inet_socket_address_new (gia, priv->port);
-
-	len = g_socket_address_get_native_size (gsa);
-	priv->sockaddr = g_malloc (len);
-	if (!g_socket_address_to_native (gsa, priv->sockaddr, len, NULL)) {
-                /* can't happen: We know the address format is supported
-                 * and the buffer is large enough */
-                g_warn_if_reached ();
-        }
-	g_object_unref (gsa);
-
-	return SOUP_STATUS_OK;
-}
-
-static guint
-update_name (SoupAddress *addr, const char *name, GError *error)
+static void
+update_address (SoupAddress *addr, SoupDNSLookup *lookup)
 {
 	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
 
-	if (error) {
-		if (error->domain == G_IO_ERROR &&
-		    error->code == G_IO_ERROR_CANCELLED)
-			return SOUP_STATUS_CANCELLED;
-		else
-			return SOUP_STATUS_CANT_RESOLVE;
-	} else if (!name)
-		return SOUP_STATUS_CANT_RESOLVE;
-	else if (priv->name)
-		return SOUP_STATUS_OK;
+	if (!priv->name)
+		priv->name = soup_dns_lookup_get_hostname (lookup);
 
-	priv->name = g_strdup (name);
-	return SOUP_STATUS_OK;
+	if (!priv->sockaddr) {
+		priv->sockaddr = soup_dns_lookup_get_address (lookup);
+		SOUP_ADDRESS_SET_PORT (priv, htons (priv->port));
+	}
 }
 
 typedef struct {
+	SoupAddress         *addr;
 	SoupAddressCallback  callback;
 	gpointer             callback_data;
 } SoupAddressResolveAsyncData;
 
 static void
-complete_resolve_async (SoupAddress *addr, guint status)
+lookup_resolved (SoupDNSLookup *lookup, guint status, gpointer user_data)
 {
-	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
-	SoupAddressResolveAsyncData *res_data;
-	GSList *lookups, *l;
+	SoupAddressResolveAsyncData *res_data = user_data;
+	SoupAddress *addr;
+	SoupAddressCallback callback;
+	gpointer callback_data;
 
-	lookups = priv->async_lookups;
-	priv->async_lookups = NULL;
+	addr = res_data->addr;
+	callback = res_data->callback;
+	callback_data = res_data->callback_data;
+	g_free (res_data);
 
-	for (l = lookups; l; l = l->next) {
-		res_data = l->data;
+	if (status == SOUP_STATUS_OK)
+		update_address (addr, lookup);
 
-		if (res_data->callback) {
-			res_data->callback (addr, status,
-					    res_data->callback_data);
-		}
-		g_slice_free (SoupAddressResolveAsyncData, res_data);
-	}
-	g_slist_free (lookups);
+	if (callback)
+		callback (addr, status, callback_data);
 
 	g_object_unref (addr);
 }
 
-static void
-lookup_resolved (GObject *source, GAsyncResult *result, gpointer user_data)
-{
-	GResolver *resolver = G_RESOLVER (source);
-	SoupAddress *addr = user_data;
-	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
-	GError *error = NULL;
-	guint status;
-
-	if (!priv->sockaddr) {
-		GList *addrs;
-
-		addrs = g_resolver_lookup_by_name_finish (resolver, result,
-							  &error);
-		status = update_addrs (addr, addrs, error);
-		g_resolver_free_addresses (addrs);
-	} else if (!priv->name) {
-		char *name;
-
-		name = g_resolver_lookup_by_address_finish (resolver, result,
-							    &error);
-		status = update_name (addr, name, error);
-		g_free (name);
-	} else
-		status = SOUP_STATUS_OK;
-
-	if (error)
-		g_error_free (error);
-
-	complete_resolve_async (addr, status);
-}
-
-static gboolean
-idle_complete_resolve (gpointer addr)
-{
-	complete_resolve_async (addr, SOUP_STATUS_OK);
-	return FALSE;
-}
-
 /**
  * SoupAddressCallback:
  * @addr: the #SoupAddress that was resolved
@@ -683,12 +577,6 @@ idle_complete_resolve (gpointer addr)
  * If @cancellable is non-%NULL, it can be used to cancel the
  * resolution. @callback will still be invoked in this case, with a
  * status of %SOUP_STATUS_CANCELLED.
- *
- * It is safe to call this more than once on a given address, from the
- * same thread, with the same @async_context (and doing so will not
- * result in redundant DNS queries being made). But it is not safe to
- * call from multiple threads, or with different @async_contexts, or
- * mixed with calls to soup_address_resolve_sync().
  **/
 void
 soup_address_resolve_async (SoupAddress *addr, GMainContext *async_context,
@@ -697,58 +585,18 @@ soup_address_resolve_async (SoupAddress *addr, GMainContext *async_context,
 {
 	SoupAddressPrivate *priv;
 	SoupAddressResolveAsyncData *res_data;
-	GResolver *resolver;
-	gboolean already_started;
 
 	g_return_if_fail (SOUP_IS_ADDRESS (addr));
 	priv = SOUP_ADDRESS_GET_PRIVATE (addr);
-	g_return_if_fail (priv->name || priv->sockaddr);
 
-	/* We don't need to do locking here because the async case is
-	 * not intended to be thread-safe.
-	 */
-
-	if (priv->name && priv->sockaddr && !callback)
-		return;
-
-	res_data = g_slice_new0 (SoupAddressResolveAsyncData);
-	res_data->callback = callback;
+	res_data = g_new (SoupAddressResolveAsyncData, 1);
+	res_data->addr          = addr;
+	res_data->callback      = callback;
 	res_data->callback_data = user_data;
 
-	already_started = priv->async_lookups != NULL;
-	priv->async_lookups = g_slist_prepend (priv->async_lookups, res_data);
-
-	if (already_started)
-		return;
-
 	g_object_ref (addr);
-
-	if (priv->name && priv->sockaddr) {
-		soup_add_completion (async_context, idle_complete_resolve, addr);
-		return;
-	}
-
-	resolver = g_resolver_get_default ();
-	if (async_context && async_context != g_main_context_default ())
-		g_main_context_push_thread_default (async_context);
-
-	if (priv->name) {
-		g_resolver_lookup_by_name_async (resolver, priv->name,
-						 cancellable,
-						 lookup_resolved, addr);
-	} else {
-		GInetAddress *gia;
-
-		gia = soup_address_make_inet_address (addr);
-		g_resolver_lookup_by_address_async (resolver, gia,
-						    cancellable,
-						    lookup_resolved, addr);
-		g_object_unref (gia);
-	}
-
-	if (async_context && async_context != g_main_context_default ())
-		g_main_context_pop_thread_default (async_context);
-	g_object_unref (resolver);
+	soup_dns_lookup_resolve_async (priv->lookup, async_context, cancellable,
+				       lookup_resolved, res_data);
 }
 
 /**
@@ -763,11 +611,6 @@ soup_address_resolve_async (SoupAddress *addr, GMainContext *async_context,
  * resolution. soup_address_resolve_sync() will then return a status
  * of %SOUP_STATUS_CANCELLED.
  *
- * It is safe to call this more than once, even from different
- * threads, but it is not safe to mix calls to
- * soup_address_resolve_sync() with calls to
- * soup_address_resolve_async() on the same address.
- *
  * Return value: %SOUP_STATUS_OK, %SOUP_STATUS_CANT_RESOLVE, or
  * %SOUP_STATUS_CANCELLED.
  **/
@@ -775,204 +618,15 @@ guint
 soup_address_resolve_sync (SoupAddress *addr, GCancellable *cancellable)
 {
 	SoupAddressPrivate *priv;
-	GResolver *resolver;
-	GError *error = NULL;
 	guint status;
 
 	g_return_val_if_fail (SOUP_IS_ADDRESS (addr), SOUP_STATUS_MALFORMED);
 	priv = SOUP_ADDRESS_GET_PRIVATE (addr);
-	g_return_val_if_fail (priv->name || priv->sockaddr, SOUP_STATUS_MALFORMED);
-
-	resolver = g_resolver_get_default ();
-
-	/* We could optimize this to avoid multiple lookups the same
-	 * way _resolve_async does, but we don't currently. So, first
-	 * lock the mutex to ensure we have a consistent view of
-	 * priv->sockaddr and priv->name, unlock it around the
-	 * blocking op, and then re-lock it to modify @addr.
-	 */
-	g_mutex_lock (priv->lock);
-	if (!priv->sockaddr) {
-		GList *addrs;
-
-		g_mutex_unlock (priv->lock);
-		addrs = g_resolver_lookup_by_name (resolver, priv->name,
-						   cancellable, &error);
-		g_mutex_lock (priv->lock);
-
-		status = update_addrs (addr, addrs, error);
-		g_resolver_free_addresses (addrs);
-	} else if (!priv->name) {
-		GInetAddress *gia;
-		char *name;
-
-		g_mutex_unlock (priv->lock);
-		gia = soup_address_make_inet_address (addr);
-		name = g_resolver_lookup_by_address (resolver, gia,
-						     cancellable, &error);
-		g_object_unref (gia);
-		g_mutex_lock (priv->lock);
-
-		status = update_name (addr, name, error);
-		g_free (name);
-	} else
-		status = SOUP_STATUS_OK;
-	g_mutex_unlock (priv->lock);
-
-	if (error)
-		g_error_free (error);
-	g_object_unref (resolver);
 
+	g_object_ref (addr);
+	status = soup_dns_lookup_resolve (priv->lookup, cancellable);
+	if (status == SOUP_STATUS_OK)
+		update_address (addr, priv->lookup);
+	g_object_unref (addr);
 	return status;
 }
-
-/**
- * soup_address_is_resolved:
- * @addr: a #SoupAddress
- *
- * Tests if @addr has already been resolved. Unlike the other
- * #SoupAddress "get" methods, this is safe to call when @addr might
- * be being resolved in another thread.
- *
- * Return value: %TRUE if @addr has been resolved.
- **/
-gboolean
-soup_address_is_resolved (SoupAddress *addr)
-{
-	SoupAddressPrivate *priv;
-	gboolean resolved;
-
-	g_return_val_if_fail (SOUP_IS_ADDRESS (addr), FALSE);
-	priv = SOUP_ADDRESS_GET_PRIVATE (addr);
-
-	g_mutex_lock (priv->lock);
-	resolved = priv->sockaddr && priv->name;
-	g_mutex_unlock (priv->lock);
-
-	return resolved;
-}
-
-/**
- * soup_address_hash_by_name:
- * @addr: a #SoupAddress
- *
- * A hash function (for #GHashTable) that corresponds to
- * soup_address_equal_by_name(), qv
- *
- * Return value: the named-based hash value for @addr.
- *
- * Since: 2.26
- **/
-guint
-soup_address_hash_by_name (gconstpointer addr)
-{
-	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
-
-	g_return_val_if_fail (priv->name != NULL, 0);
-	return g_str_hash (priv->name);
-}
-
-/**
- * soup_address_equal_by_name:
- * @addr1: a #SoupAddress with a resolved name
- * @addr2: another #SoupAddress with a resolved name
- *
- * Tests if @addr1 and @addr2 have the same "name". This method can be
- * used with soup_address_hash_by_name() to create a #GHashTable that
- * hashes on address "names".
- *
- * Comparing by name normally means comparing the addresses by their
- * hostnames. But if the address was originally created using an IP
- * address literal, then it will be compared by that instead.
- *
- * In particular, if "www.example.com" has the IP address 10.0.0.1,
- * and @addr1 was created with the name "www.example.com" and @addr2
- * was created with the name "10.0.0.1", then they will compare as
- * unequal for purposes of soup_address_equal_by_name().
- *
- * This would be used to distinguish hosts in situations where
- * different virtual hosts on the same IP address should be considered
- * different. Eg, for purposes of HTTP authentication or cookies, two
- * hosts with the same IP address but different names are considered
- * to be different hosts.
- *
- * See also soup_address_equal_by_ip(), which compares by IP address
- * rather than by name.
- *
- * Return value: whether or not @addr1 and @addr2 have the same name
- *
- * Since: 2.26
- **/
-gboolean
-soup_address_equal_by_name (gconstpointer addr1, gconstpointer addr2)
-{
-	SoupAddressPrivate *priv1 = SOUP_ADDRESS_GET_PRIVATE (addr1);
-	SoupAddressPrivate *priv2 = SOUP_ADDRESS_GET_PRIVATE (addr2);
-
-	g_return_val_if_fail (priv1->name != NULL, FALSE);
-	g_return_val_if_fail (priv2->name != NULL, FALSE);
-	return !g_ascii_strcasecmp (priv1->name, priv2->name);
-}
-
-/**
- * soup_address_hash_by_ip:
- * @addr: a #SoupAddress
- *
- * A hash function (for #GHashTable) that corresponds to
- * soup_address_equal_by_ip(), qv
- *
- * Return value: the IP-based hash value for @addr.
- *
- * Since: 2.26
- **/
-guint
-soup_address_hash_by_ip (gconstpointer addr)
-{
-	SoupAddressPrivate *priv = SOUP_ADDRESS_GET_PRIVATE (addr);
-	guint hash;
-
-	g_return_val_if_fail (priv->sockaddr != NULL, 0);
-
-	memcpy (&hash, SOUP_ADDRESS_GET_DATA (priv),
-		MIN (sizeof (hash), SOUP_ADDRESS_FAMILY_DATA_SIZE (priv->sockaddr->sa_family)));
-	return hash;
-}
-
-/**
- * soup_address_equal_by_ip:
- * @addr1: a #SoupAddress with a resolved IP address
- * @addr2: another #SoupAddress with a resolved IP address
- *
- * Tests if @addr1 and @addr2 have the same IP address. This method
- * can be used with soup_address_hash_by_ip() to create a
- * #GHashTable that hashes on IP address.
- *
- * This would be used to distinguish hosts in situations where
- * different virtual hosts on the same IP address should be considered
- * the same. Eg, if "www.example.com" and "www.example.net" have the
- * same IP address, then a single #SoupConnection can be used to talk
- * to either of them.
- *
- * See also soup_address_equal_by_name(), which compares by name
- * rather than by IP address.
- *
- * Return value: whether or not @addr1 and @addr2 have the same IP
- * address.
- *
- * Since: 2.26
- **/
-gboolean
-soup_address_equal_by_ip (gconstpointer addr1, gconstpointer addr2)
-{
-	SoupAddressPrivate *priv1 = SOUP_ADDRESS_GET_PRIVATE (addr1);
-	SoupAddressPrivate *priv2 = SOUP_ADDRESS_GET_PRIVATE (addr2);
-	int size;
-
-	g_return_val_if_fail (priv1->sockaddr != NULL, FALSE);
-	g_return_val_if_fail (priv2->sockaddr != NULL, FALSE);
-
-	size = SOUP_ADDRESS_FAMILY_SOCKADDR_SIZE (priv1->sockaddr->sa_family);
-	return (priv1->sockaddr->sa_family ==
-		priv2->sockaddr->sa_family &&
-		!memcmp (priv1->sockaddr, priv2->sockaddr, size));
-}
