@@ -32,7 +32,6 @@ typedef enum {
 	SOUP_STATUS_IO_ERROR,
 	SOUP_STATUS_MALFORMED,
 	SOUP_STATUS_TRY_AGAIN,
-	SOUP_STATUS_TOO_MANY_REDIRECTS,
 
 	/* HTTP Status Codes */
 	SOUP_STATUS_CONTINUE                        = 100,
@@ -93,7 +92,6 @@ typedef enum {
 } SoupKnownStatusCode;
 
 const char *soup_status_get_phrase (guint status_code);
-guint       soup_status_proxify    (guint status_code);
 
 #define SOUP_HTTP_ERROR soup_http_error_quark()
 GQuark soup_http_error_quark (void);
